// 工具函数

/**
 * 控制台打印
 * @param object 待打印对象
 */
function print(object) {
    console.log(object);
}

/**
 * 深复制一个JSON对象
 * @param jsonObj JSON对象
 * @returns 拷贝的JSON对象
 */
function DeepCloneJsonObj(jsonObj) {
    return JSON.parse(JSON.stringify(jsonObj));
}

// AJAX全局设置

// 设置AJAX请求为同步
function SetSynchronizedAjaxRequest() {
    $.ajaxSetup({
        async: false
    });
}

// 恢复AJAX请求为同步
function SetAsynchronizedAjaxRequest() {
    $.ajaxSetup({
        async: true
    });
}

// 设置缓存
function SetCachedScript() {
    $.ajaxSetup({ 
        cache: true 
    }); 
}

// 设置为不缓存
function SetUnachedScript() {
    $.ajaxSetup({ 
        cache: false 
    }); 
}

/**
 * 判断jQuery对象是否存在
 * @param jQueryObj jQuery对象
 * @returns jQuery对象存在返回true
 */
function IsJqObjExists(jQueryObj) {
    return jQueryObj.length > 0;
}

// 控件设置

/**
 * 确认、取消对话框 弹出对话框之后监听其#message-dialog-modal-confirm(确认)、
 * #message-dialog-modal-cancle(取消)的click事件
 * @param message 对话框消息内容
 * @param caption 对话框标题
 * @param onConfirm 确认的回调函数
 * @param onCancle 取消的回调函数
 */
function MessageDialog(message, caption, onConfirm, onCancle) {
    if (caption == null) {
        caption = '消息提示';
    }
    
    var messageDialogObj = $('#modal-control-message-dialog');
    messageDialogObj.modal({
        backdrop : 'static',
        keyboard : false
    });
    
    $('#modal-control-caption-message-dialog').text(caption);
    $('#modal-control-body-message-dialog').html(message);
    messageDialogObj.modal('show');
    
    // 注册回调函数
    var confirmActionObj = $('#modal-control-action-confirm-message-dialog');
    confirmActionObj.unbind('click');
    confirmActionObj.click(onConfirm);
    
    var cancelActionObj = $('#modal-control-action-cancel-message-dialog');
    cancelActionObj.unbind('click');
    cancelActionObj.click(onCancle);
}

/**
 * 消息提示框，取代alert
 * @param message 消息
 * @param caption 消息框标题
 */
function MessageBox(message, caption) {
    if (caption == null) {
        caption = '消息提示';
    }
    
    $('#modal-control-caption-message-box').text(caption);
    $('#modal-control-body-message-box').html(message);
    $('#modal-control-message-box').modal('show');
}

/**
 * 评论对话框
 * @param caption 评论框标题
 * @param message 默认消息
 * @param onConfirm 点击确认回调函数
 * @param onCancle 点击取消回调函数
 */
function CommentDialog(caption, message, onConfirm, onCancle) {
    if (caption == null) {
        caption = "请输入您的评论或备注";
    }
    
    if (message == null) {
        message = "";
    }
    
    var commentDialogObj = $('#modal-control-comment-dialog');
    commentDialogObj.modal({
        backdrop : 'static',
        keyboard : false
    });
    
    $('#modal-control-caption-comment-dialog').text(caption);
    $('#modal-control-body-comment-dialog').text(message);
    commentDialogObj.modal('show');
    
    // 注册消息处理
    var confirmActionObj = $('#modal-control-button-confirm-comment-dialog');
    confirmActionObj.unbind('click');
    confirmActionObj.click(onConfirm);
    
    var cancelActionObj = $('#modal-control-button-cancel-comment-dialog');
    cancelActionObj.unbind('click');
    cancelActionObj.click(onCancle);
}

/**
 * 获取评论框中的内容
 * @returns 评论框中的内容
 */
function GetCommentDialogText() {
    return $('#modal-control-body-comment-dialog').val();
}

/**
 * 文件上传进度
 * @param formObj 要上传文件的表单对象
 * @param fileUploadUrl 上传路径
 * @param onComplete 上传处理完成
 * @param caption 进度条标题
 */
function UploadProgresBox(formObj, fileUploadUrl, onComplete, caption) {
    // 完成进度条
    var completeProgressObj = $('#file-upload-progress-bar-complete-progress');
    // 完成比例
    var completePercentTxtObj = $('#file-upload-progress-bar-modal-complete-percent');
    // 进度条模态框
    var fileUploadProgressBarModalObj = $('#modal-control-progress-bar');
    
    // 上传标题
    if (caption == null || caption == '') {
        caption = '附件上传中, 请稍后...';
    }
    $('#modal-control-caption-progress-bar').text(caption);
    
    var showProgressBar = false;
    
    formObj.ajaxSubmit({
        // 上传路径
        url: fileUploadUrl,
        // 返回JSON
        dataType: 'json',
        // 发送前
        beforeSend: function() {
                // 获取文件对象
                var fileObj = $('input[type=file]');
                
                // 文件大小检测
                // 超过多大则显示文件上传进度条
                showProgressBar = GetFileSize(fileObj) > 0;
                
                if (showProgressBar) {
                    // 显示进度条
                    fileUploadProgressBarModalObj.modal('show');
                    
                    var percentVal = '0%';
                    completeProgressObj.width(percentVal);
                    completePercentTxtObj.text(percentVal);
                }
        },
        // 发送进度处理
        uploadProgress: function(event, position, total, percentComplete) {
                if (showProgressBar) {
                    // 显示进度条
                    var percentVal = percentComplete + '%';
                    completeProgressObj.width(percentVal);
                    completePercentTxtObj.text(percentVal);
            }
        },
        // 上传成功
        success: function() {
                if (showProgressBar) {
                    // 关闭进度条
                    fileUploadProgressBarModalObj.modal('hide');
            }
        },
        // 上传完成
        complete: onComplete,
        // 30秒超时
        timeout: 180000
    });
}

// 获取上传文件大小
function GetFileSize(jqFileObj) {
    var fileSize = 0;
    
    try {
        // 需要HTML支持
        fileSize = jqFileObj[0].files[0].size;
    } catch (e) {
        fileSize = -1;
    }
    
    return fileSize;
}

/**
 * 显示日期选择器
 * @returns 无
 */
function ShowDatePickerHandler() {
    RegisterLayDateControlHandler('YYYY/MM/DD', ".datepicker-control");
}

/**
 * 日期时间选择器注册
 * @returns 无
 */
function ShowDateTimePickerHandler() {
    RegisterLayDateControlHandler('YYYY/MM/DD hh:mm:ss', ".datetimepicker-control");
}

/**
 * laydate控件注册
 * @param dateFormat 日期格式
 * @param elementSelector 监听元素选择器
 * @returns 无
 */
function RegisterLayDateControlHandler(dateFormat, elementSelector) {
    if (dateFormat == null || elementSelector == null) {
        return;
    }
    
    // 时间控件
    $(document).on("click", elementSelector, function() {
        laydate({istime: true, format: dateFormat});
    });
}

/**
 * 下拉列表框处理器
 * @param dataObjs 显示的数据对象
 * @param ctrlButtonId 用于点击的控件ID
 * @param validClass 有效的类名(如果点击不发生在含该类的元素上，列表将被关闭)
 * @param displayDivId 用于显示列表的外层元素
 * @param hideListCallback 隐藏列表的方法
 * @param onItemCreated 元素创建后、添加到类表中之前的回调函数
 * @param onItemSelected 列表项被选中时处理函数
 */
function ListControlHandler(dataObjs, ctrlButtonId, validClass, 
        displayDivId, onItemCreated, onItemSelected, hideListCallback) {
    var func = function() {
        // 数据项原型
        var liObjPrototype = $('<li class="list-group-item"></li>');
        
        // 显示教师的容器对象
        var containerObj = $('#' + displayDivId);
        var containerListObj = containerObj.children(".list-group");
        containerListObj.html("");
        
        $.each(dataObjs, function(index, dataObj){
            // 浅复制节点
            var liObj = liObjPrototype.clone();
            liObj.addClass(validClass);
            
            // 创建元素前(传引用)
            onItemCreated(liObj, dataObj, index);
            
            containerListObj.append(liObj);
        });
        containerObj.slideDown();
        
        // 如果未传入关闭函数
        if (hideListCallback == null) {
            hideListCallback = function(){
                containerObj.slideUp();
            };
        }
        
        // 如果未处理点击事件，则隐藏
        if (onItemSelected == null) {
            onItemSelected = hideListCallback;
        }
        
        // 注册选择点击列表外事件处理器
        OuterClickHandler(validClass, hideListCallback);
        
        // 注册点击列表事件
        containerListObj.children('.list-group-item').click(onItemSelected);
    };
    
    // 点击触发，或至直接使用触发
    if (ctrlButtonId != null) {
        $(document).on('click', '#' + ctrlButtonId, func);
    } else {
        func();
    }
}

/**
 * 注册点击外部选择框时的点击事件
 * @param className 含有className的class值
 * @param onFailed 如果没有该属性值，回调该函数
 */
function OuterClickHandler(className, onFailed){
    // 点击外侧关闭下拉列表
    $(document).click(function(e) {
        var eventTarget = $(e.target);
        if (!eventTarget.hasClass(className)) {
            onFailed();
        }
    });
}

/**
 * 设置选择控件jQuery对象selectCtrlObj选中项为value, value存在于selectCtrlObj中
 * @param selectCtrlObj 选择控件jQuery对象
 * @param value 待选中的值
 * @return boolean 选中成功返回true, 否则返回false
 */
function SetSelectControlSelected(selectCtrlObj, value) {
    if (selectCtrlObj == null || value == null) {
        return false;
    }
    
    selectCtrlObj.val(value);
    
    return true;
}

/**
 * 设置选择控件jQuery对象selectCtrlObj选中项为显示为name, 值为value, value存在于selectCtrlObj中
 * @param selectCtrlObj 选择控件jQuery对象
 * @param value 待选中的值
 * @param name 带显示的选中名
 * @returns boolean 设置成功返回true，否则返回false
 */
function SetSelectControlDisplay(selectCtrlObj, value, name) {
    var selectedObj = selectCtrlObj.find("option:selected");
    selectedObj.val(value);
    selectedObj.text(name);
}

/**
 * 设定当选按钮选中项
 * @param radioItems input对象数组
 * @param value 待选中的值
 * @returns {Boolean}
 */
function SetCheckedRadio(radioItems, value) {
    $.each(radioItems, function(index, radioItem) {
        var radioItemObj = $(radioItem);
        
        if (radioItemObj.val() == value) {
            radioItemObj.prop('checked', true);
            return true;
        }
    });
    
    return false;
}

/**
 * 获取选中项的文本内容
 * @param radioItems 单选按钮的jQuery对象
 * @returns 选中项的文本内容
 */
function GetCheckedRadioText(radioItems) {
    var radioItemObj;
    
    $.each(radioItems, function(index, radioItem) {
        radioItemObj = $(radioItem);
        
        if (radioItemObj.prop('checked')) {
            return false;
        }
    });

    return radioItemObj;
}

/**
 * 设置复选框选中事件
 * @param checkboxName 复选框name值
 * @param checkedValues 需要选中的值
 * @returns 无
 */
function SetCheckboxSelected(checkboxName, checkedValues) {
    // 取消当前选择
    var checkboxCtrls = $('[type="checkbox"][name="' + checkboxName + '"]');
    checkboxCtrls.prop('checked', false);
    
    // 设置待选中值
    checkboxCtrls.each(function(){
        var thisObj = $(this);
        $.each(checkedValues, function(idx, checkedValue){
            if (checkedValue == thisObj.val()) {
                thisObj.prop('checked', true);
                return false;
            }
        });
    });
}

/**
 * 获取选中的复选框的值
 * @param checkboxName 复选框name值
 * @returns 数组
 */
function GetSelectedCheckbox(checkboxName) {
    // 复选框
    var checkedValues = [];
    $('[type="checkbox"][name="' + checkboxName + '"]:checked').each(function(){
        checkedValues.push($(this).val());
    });
    
    return checkedValues;
}

// 错误处理

/**
 * 判断是否有错误发生
 * @param errno 错误编号
 * @returns {Boolean} 无错误返回true
 */
function IsError(errno){
    return errno == undefined || errno != ERRNO_NONE;
}

/**
 * 获取出错消息内容
 * @param errno 错误编号
 * @param error 错误描述
 * @returns {String}
 */
function GetErrorMessage(errno, error) {
    return "错误代码: " + errno + ", 错误描述: " + error;
}

/**
 * 默认错误处理函数
 * @param errno 错误编号
 * @param error 错误描述
 * @returns {Boolean}
 */
function ErrorHandler(errno, error){
    MessageBox("处理发生了错误， " + GetErrorMessage(errno, error));
    
    return false;
}

// 数据处理

/**
 * 检查登录是否有效
 * @param json XMLHttpRequest结果
 * @param message 消息
 * @param caption 提示框标题
 * @returns {Boolean}
 */
function CheckResponse(json, message, caption) {
    if (!json || json.errno == undefined) {
        if (message == null) {
            message = "请求未响应，请刷新页面";
        }
        
        if (caption == null) {
            caption = "刷新页面提示";
        }
        
        // 提示
        MessageBox(message, caption);
        
        return false;
    }
    
    return true;
}

/**
 * 
 * 发送数据(进行登录检测)
 * @param url 数据发送URL
 * @param data(JSON数据且必须不未false)
 * @param onSuccess 成功是执行回调 传入jsonObj
 * @param onFailed 失败是执行回调 传入errno、error、及完整地json对象
 * @returns {Boolean}
 */
function PostData(url, data, onSuccess, onFailed){
    if (!data) {
        return false;
    }
    
    $.post(url, data, function(jsonObj){
        if (CheckResponse(jsonObj)) {
            if (IsError(jsonObj.errno)) {
                if (onFailed != null) {
                    onFailed(jsonObj.errno, jsonObj.error, jsonObj);
                } else {
                    ErrorHandler(jsonObj.errno, jsonObj.error);
                }
                
                return false;
            }
            
            if (onSuccess != null) {
                onSuccess(jsonObj);
            }
            
            return true;
        }
        
        return false;
    });
}

/**
 * 加载内容
 * @param containerObj 容器的jQuery对象
 * @param url 获取内容的Url
 */
function LoadContent(containerObj, url) {
    $.get(url, function(content) {
        containerObj.append(content);
    });
}

/**
 * 加载HTML页面代码
 * @param requestUrl 请求链接
 * @param onSuccess 加载成功后的处理函数
 * @param onFailed 加载失败后的处理函数
 * @returns 无
 */
function LoadHtml(requestUrl, onSuccess, onFailed) {
    $.get(requestUrl, {}, function(htmlContent, status, xmlHttpRequestObj) {
        if (status == "success") {
            if (onSuccess != null) {
                onSuccess(htmlContent);
            }
            
            return;
        }
        
        if (onFailed != null) {
            onFailed(htmlContent, status, xmlHttpRequestObj);
        }
    }, "html");
}

/**
 * 加载JS脚本
 * @param url 脚本地址
 * @param onSuccess 加载成功后执行
 * @param onFailed 加载失败后执行
 */
function LoadScript(url, onSuccess, onFailed) {
    if (onFailed == null) {
        onFailed = function() {
            MessageBox("网络连接错误");
        }
    }
    
    $.getScript(url).done(onSuccess).fail(onFailed);
}

// 数字处理

/**
 * 判断是否为正数
 * @param num 数字
 * @returns 是正数返回true
 */
function isPositiveNumber(num) {
    return !isNaN(num) && num > 0;
}

/**
 * 判断是否为非正数
 * @param num 数字
 * @returns 为非正数返回true
 */
function isNonPositiveNumber(num) {
    return !isNaN(num) && num <= 0;
}

/**
 * 判断是否为负数
 * @param num 数字
 * @returns 为负数返回true
 */
function isNegativeNumber(num) {
    return !isNaN(num) && num < 0;
}

/**
 * 判断是否为非负数
 * @param num 数字
 * @returns 为非负数返回true
 */
function isNonNegativeNumber(num) {
    return !isNaN(num) && num >= 0;
}

// 时间处理

/**
 * 获取当前系统时间毫秒数
 * @returns 系统时间毫秒数
 */
function GetCurrentMilliSeconds() {
    return parseInt(new Date().getTime());
}

// 获取当前UNIX时间戳
function GetCurrentTime() {
    return parseInt(GetCurrentMilliSeconds() /1000);
}

/**
 * 格式化日期
 * @param timestamp UNIX时间戳
 * @returns {String} 形如 2016/11/08
 */
function formatDate(timestamp) {
    if (timestamp == null || timestamp == 0) {
        timestamp = GetCurrentTime();
    }
    
    return new Date(timestamp * 1000).format("yyyy/MM/dd");
}

/**
 * 格式化日期时间
 * @param timestamp UNIX时间戳
 * @returns {String} 形如 2016/11/08 12:39:31
 */
function formatDateTime(timestamp) {
    if (timestamp == null || timestamp == 0) {
        timestamp = GetCurrentTime();
    }
    
    return formatDate(timestamp) + ' ' + (new Date(timestamp * 1000).format("hh:mm:ss"));
}

/**
 * 判断是否为日期格式
 * @param dateStr 日期字符串
 * @returns 为日期格式返回true
 */
function isDate(dateStr) {
    return validateDate(dateStr);
}

/**
 * 判断是否为日期格式
 * @param dateStr 日期字符串
 * @returns 为日期格式返回true
 */
function validateDate(dateStr) {
    return REGEX_DATE.test(dateStr);
}

/**
 * 判断是否为日期时间格式
 * @param dateTimeStr 日期时间字符串
 * @returns 为日期格式返回true
 */
function isDateTime(dateTimeStr) {
    return validateDateTime(dateTimeStr);
}

/**
 * 判断是否为日期时间格式
 * @param dateTimeStr 日期时间字符串
 * @returns 为日期格式返回true
 */
function validateDateTime(dateTimeStr) {
    return REGEX_DATETIME.test(dateTimeStr);
}

/**
 * 解析时间返回时间戳
 * @param dateTimeStr yyyy/MM/dd hh:mm:ss形式
 * @returns {Number}
 */
function parseDateTime(dateTimeStr) {
    if (!REGEX_DATETIME.test(dateTimeStr)) {
        return TIMESTAMP_DEFAULT;
    }
    
    return (Date.parse(new Date(dateTimeStr)) / 1000);
}

/**
 * 解析时间返回时间戳(00:00:00)
 * @param dateStr yyyy/MM/dd形式
 * @returns {Number}
 */
function parseDate(dateStr) {
    return parseDateStart(dateStr);
}

/**
 * 
 * 解析时间返回时间戳(00:00:00)
 * @param dateStr yyyy/MM/dd形式
 * @returns {Number}
 */
function parseDateStart(dateStr) {
    if (!REGEX_DATE.test(dateStr)) {
        return TIMESTAMP_DEFAULT;
    }
    
    return parseDateTime(dateStr + ' 00:00:00');
}

/**
 * 将日期解析称时间戳(23:59:59)
 * @param dateStr yyyy/MM/dd形式
 * @returns {Number}
 */
function parseDateEnd(dateStr) {
    if (!REGEX_DATE.test(dateStr)) {
        return TIMESTAMP_DEFAULT;
    }
    
    return parseDateTime(dateStr + ' 23:59:59');
}

//对Date的扩展，将 Date 转化为指定格式的String 
//月(M)、日(d)、小时(h)、分(m)、秒(s)、季度(q) 可以用 1-2 个占位符， 
//年(y)可以用 1-4 个占位符，毫秒(S)只能用 1 个占位符(是 1-3 位的数字) 
//(new Date()).Format("yyyy-MM-dd hh:mm:ss.S") ==> 2006-07-02 08:09:04.423 
//(new Date()).Format("yyyy-M-d h:m:s.S")      ==> 2006-7-2 8:9:4.18 
Date.prototype.format = function(fmt) {
    //author: meizz
    var o = { 
            "M+" : this.getMonth()+1,                 //月份
            "d+" : this.getDate(),                    //日
            "h+" : this.getHours(),                   //小时
            "m+" : this.getMinutes(),                 //分
            "s+" : this.getSeconds(),                 //秒
            "q+" : Math.floor((this.getMonth()+3)/3), //季度
            "S"  : this.getMilliseconds()             //毫秒
    };
    
    if(/(y+)/.test(fmt)) {
        fmt=fmt.replace(RegExp.$1, (this.getFullYear()+"").substr(4 - RegExp.$1.length)); 
    }
    for(var k in o) {
        if(new RegExp("("+ k +")").test(fmt)) {
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length))); 
        }
    }
    
    return fmt; 
};

// 字符串处理

/**
 * 获取默认显示内容
 * @param content 内容
 * @returns 原始内容或者"-"
 */
function GetDefault(content) {
    return content == null ? "-" : content;
}

/**
 * 检查输入框是否不为空(去左右空白)
 * @param inputJqObj 输入框jQuery对象
 * @param onSuccess 验证成功回调
 * @param onError 验证失败回调
 * @returns 成功返回值，失败返回false
 */
function validateNotEmpty(inputJqObj, onSuccess, onError) {
    var val = inputJqObj.val();
    if (val != null && val.trim() == '') {
        if (onError != null) {
            onError();
        }
        
        inputJqObj.focus();
        return false;
    }
    
    if (onSuccess != null) {
        onSuccess();
    }
    return val.trim();
}

/**
 * 判断是否为正则格式
 * @param email 邮件地址
 * @returns 格式正确返回true
 */
function validateEmail(email) {
    return REGEX_EMAIL_GENERIC.test(email);
}

/**
 * 将方括号字符串转为数组
 * @param bracketStr 方括号字符串转为数组，如[pdf, docx]
 * @returns 数组
 */
function ExtractBracketStrToArray(bracketStr){
    if (!REGEX_SQUARE_BRACKET.test(bracketStr)) {
        return [];
    }
    
    bracketStr = bracketStr.replace("[", "");
    bracketStr = bracketStr.replace("]", "");
    bracketStr = bracketStr.replaceAll(",\\s*", ",");
    
    return bracketStr.split(",");
}

/**
 * 判断字符串content是否包含subject子串
 * @param content 被判断字符串
 * @param subject 子串
 * @returns 如果字符串content包含子串subject返回true, 否则返回false
 */
function hasText(content, subject) {
    return content != null && content != undefined && content.indexOf(subject) != -1;
}

/**
 * 替换所有匹配内容
 */
String.prototype.replaceAll = function(regexStr, replacement) {
    return this.replace(new RegExp(regexStr, 'g'), replacement);
};

// 位置、坐标
/**
 * 获取元素大小(二维)
 * @param eleObj 待获取元素
 * @returns 含宽度和高度的JSON数据
 */
function getElementSize(eleObj) {
    var sizeInfo = {
        "width" : eleObj.width(),
        "height" : eleObj.height()
    };

    return sizeInfo;
}

/**
 * 获取点击点的绝对坐标
 * @param eleObj 点击的元素
 * @param event 点击事件对象
 * @returns 点击元素的绝对坐标
 */
function getClickCoordinate(eleObj, event) {
    var coordinateInfo = {
        'x' : event.pageX,
        'y' : event.pageY
    };
    
    return coordinateInfo;
}

/**
 * 获取点击相对元素坐标(二维)
 * @param eleObj 点击的元素
 * @param event 点击事件对象
 * @returns 点击元素的相对坐标
 */
function getClickRelCoordinate(eleObj, event) {
    var startX = eleObj.offset().left;
    var startY = eleObj.offset().top;

    var x = event.pageX;
    var y = event.pageY;

    var coordinateInfo = {
        'x' : x - startX,
        'y' : y - startY
    };

    return coordinateInfo;
}
